/**************************************************************************
*
*	DumbRobot.cs
*
*	Main source file for dumb robot sample programme (in C#)
*
*	Author: Saxon Druce
*
*	Copyright (c) 2004 Block Software Pty Ltd
*
**************************************************************************/

using System;
using System.Collections;
using ControllerNet;
using VecNet;

namespace SimpleRobotNet
{
	/// <summary>
	/// The main class.
	/// </summary>
	class SimpleRobot
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			try
			{
				// setup default values of parameters
				string serverName = "localhost";

				// expect arguments to be in the following format:
				// /a a_value /b b_value
				// so there should be an even number of them
				if (args.Length % 2 != 0)
					throw new Exception("Expected an even number of arguments");

				// now loop over them
				int i;
				for (i = 0; i < args.Length; i += 2)
				{
					if (args[i] == "/s")
					{
						// /s = server address (a string)
						serverName = args[i + 1];
					}
					else
					{
						throw new Exception("Invalid argument " + args[i]);
					}
				}

				// create a controller
				Controller controller = new Controller();

				// connect
				controller.Connect(serverName, 9386);

				// create a robot
				int robotID = controller.CreateRobot();
				Console.WriteLine("Robot ID is {0}", robotID);

				// set name and owner
				controller.SetName("Dumb Robot C#");
				controller.SetOwner("Saxon");

				// get initial amount of money
				Console.WriteLine("Robot has ${0} to spend on equipment", 
					controller.GetMoney());

				// buy equipment
				controller.PurchaseEngine("Engine.2.50");
				controller.PurchaseSensor("Sensor.10.60.40");
				controller.PurchaseBattery("Battery.50");
				controller.PurchaseTurret("Turret.60.50");
				controller.PurchaseWeapon("Weapon.2.4.2.3");
				controller.PurchaseSolarPanel("SolarPanel.1");

				// get amount of money left
				Console.WriteLine("Robot has ${0} remaining after purchasing equipment", 
					controller.GetMoney());

				// enter the simulation
				controller.EnterSimulation();

				// set throttle to full speed ahead
				controller.SetThrottle(100.0);

				// loop forever, since there doesn't seem to be a way
				// to loop until a key is pressed
				Console.WriteLine("\nUse ctrl+C to stop\n");
				for (;;)
				{
					// set desired orientation to 90 degrees further clockwise
					// than we currently are, which should result in a constant
					// clockwise spin as fast as possible (assuming we aren't
					// able to spin by 90 degrees between iterations of this loop)
					controller.SetDesiredOrientation(
						controller.GetOrientation() + Math.PI / 2);

					// do the same thing with the sensor orientation, to make it
					// constantly spin (not that we do anything with it!)
					controller.SetDesiredSensorOrientation(
						controller.GetSensorOrientation() + Math.PI / 2);

					// set the desired turret orientation to a function which will
					// make it move back and forth over its range
					controller.SetDesiredTurretOrientation(
						30 * Math.Pow(Math.Sin(DateTime.Now.Ticks / 5000000.0), 3) * 
						Math.PI / 180.0);

					// fire the weapon if it's fully charged, wherever we happen
					// to be pointing
					if (controller.GetWeaponCharge() >= 100)
						controller.FireWeapon();
				}
			}
			catch (Exception e)
			{
				Console.WriteLine(
					"An exception occurred: {0}",
					e.Message);
			}
		}
	}
}
